# Get Field Mapping /获取字段映射

原文链接 : [https://www.elastic.co/guide/en/elasticsearch/reference/current/indices-get-field-mapping.html](https://www.elastic.co/guide/en/elasticsearch/reference/current/indices-get-field-mapping.html)

译文链接 : [http://www.apache.wiki/display/Elasticsearch](http://www.apache.wiki/display/Elasticsearch)[（修改该链接为 ApacheCN 对应的译文链接）](https://www.elastic.co/guide/en/elasticsearch/reference/current/indices-get-mapping.html)

贡献者 : [Le-Mon](/display/~tanwen)

The get field mapping API allows you to retrieve mapping definitions for one or more fields. This is useful when you do not need the complete type mapping returned by the [_Get Mapping_](https://www.elastic.co/guide/en/elasticsearch/reference/current/indices-get-mapping.html "Get Mapping") API.

The following returns the mapping of the field `text` only:

```
curl -XGET 'localhost:9200/twitter/_mapping/tweet/field/message?pretty'
```

For which the response is (assuming `text` is a default string field):

```
{
   "twitter": {
      "mappings": {
         "tweet": {
            "message": {
               "full_name": "message",
               "mapping": {
                  "message": {
                     "type": "text",
                     "fields": {
                        "keyword": {
                           "type": "keyword",
                           "ignore_above": 256
                        }
                     }
                  }
               }
            }
         }
      }
   }
}
```

### Multiple Indices, Types and Fields

The get field mapping API can be used to get the mapping of multiple fields from more than one index or type with a single call. General usage of the API follows the following syntax: `host:port/{index}/{type}/_mapping/field/{field}` where `{index}`, `{type}` and `{field}` can stand for comma-separated list of names or wild cards. To get mappings for all indices you can use `_all` for `{index}`. The following are some examples:

```
curl -XGET 'localhost:9200/twitter,kimchy/_mapping/field/message?pretty'
curl -XGET 'localhost:9200/_all/_mapping/tweet,book/field/message,user.id?pretty'
curl -XGET 'localhost:9200/_all/_mapping/tw*/field/*.id?pretty'

```

### Specifying fields

he get mapping api allows you to specify one or more fields separated with by a comma. You can also use wildcards. The field names can be any of the following:

| Full names | the full path, including any parent object name the field is part of (ex. `[user.id](http://user.id)`). |
| Field names | the name of the field without the path to it (ex. `id` for `{ "user" : { "id" : 1 } }`). |

The above options are specified in the order the `field` parameter is resolved. The first field found which matches is returned. This is especially important if index names or field names are used as those can be ambiguous.

For example, consider the following mapping:

To select the `id` of the `author` field, you can use its full name `[author.id](http://author.id)`. `name` will return the field `[author.name](http://author.name)`:

```
curl -XGET "http://localhost:9200/publications/_mapping/article/field/author.id,abstract,name"

```

returns:

```
{
   "publications": {
      "article": {
         "abstract": {
            "full_name": "abstract",
            "mapping": {
               "abstract": { "type": "text" }
            }
         },
         "author.id": {
            "full_name": "author.id",
            "mapping": {
               "id": { "type": "text" }
            }
         },
         "name": {
            "full_name": "author.name",
            "mapping": {
               "name": { "type": "text" }
            }
         }
      }
   }
}

```

Note how the response always use the same fields specified in the request as keys. The `full_name`in every entry contains the full name of the field whose mapping were returned. This is useful when the request can refer to to multiple fields.

### Other options

| `include_defaults` | adding `include_defaults=true` to the query string will cause the response to include default values, which are normally suppressed. |